package tencentmeeting

import (
	"crypto/hmac"
	"crypto/sha256"
	"encoding/base64"
	"encoding/hex"
	"fmt"
	"math/rand"
	"net/http"
	"strings"
	"time"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/api"
	"shylinux.com/x/community/src/gonganxitong"
	"shylinux.com/x/community/src/renzhengshouquan"
	"shylinux.com/x/community/src/renzhengshouquan/external/tencentmeeting/model"
)

type tencentmeeting struct {
	renzhengshouquan.Table
	order  string `data:"12"`
	role   string `data:"leader"`
	fields string `data:"auth_uid,user_uid,app_id,sdk_id,secret_id,secret_key,user_id"`
	create string `name:"create app_id* sdk_id* secret_id* secret_key* user_id*" role:"leader"`
	remove string `name:"remove" role:"leader"`
	list   string `name:"list auth_uid uid auto" role:"leader"`
	config string `name:"config user_id:select" style:"notice" role:"leader"`
}

func (s tencentmeeting) Init(m *ice.Message, arg ...string) {
	gonganxitong.MeetingVendorAdd(m.PrefixKey(), s)
	s.Table.Init(m, arg...)
}
func (s tencentmeeting) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.USER_ID:
		s.UserList(m, m.Option(model.AUTH_UID)).Cut(arg[0], "username")
	}
}
func (s tencentmeeting) List(m *ice.Message, arg ...string) {
	if len(arg) == 1 {
		s.ValueList(m, arg).Display("")
		kit.If(m.Length() > 0, func() { m.Action() })
	} else {
		m.Cmdy(api.RENZHENGSHOUQUAN_AUTH, arg[0]).Table(func(value ice.Maps) {
			if renzhengshouquan.AuthStatus(kit.Int(value[model.AUTH_STATUS])) == renzhengshouquan.AuthIssued {
				m.PushButton(s.Config)
			} else {
				m.PushButton()
			}
		})
	}
}
func (s tencentmeeting) Config(m *ice.Message, arg ...string) {
	m.Cmdy(api.GONGANXITONG_MEETING, s.Config, m.OptionSimple(model.PLACE_UID, model.USER_UID, model.USER_ID), model.VENDOR, m.PrefixKey())
}
func (s tencentmeeting) UserList(m *ice.Message, arg ...string) *ice.Message {
	s.spide(m, "", "users/list", "operator_id", "$user_id", "operator_id_type", "1", "page", "1", "page_size", "10")
	s.parseList(m, "users", "role_code", "role_name", "userid", "username", "avatar_url", "status", "account_version", "user_account_type")
	m.RenameAppend("userid", model.USER_ID, "avatar_url", model.AVATAR)
	return m
}
func (s tencentmeeting) MeetingList(m *ice.Message, arg ...string) *ice.Message {
	s.spide(m, "", "meetings", "userid", "$user_id", "instanceid", "1", "meeting_code", kit.Select("", arg, 0))
	s.parseList(m, "meeting_info_list", "subject", "meeting_code", "start_time", "end_time", "join_url", "meeting_id")
	m.RenameAppend("meeting_code", model.UID, "join_url", model.LINK)
	m.Table(func(value ice.Maps) {
		d := (kit.Time(value["end_time"]) - kit.Time(value["start_time"])) / int64(time.Second)
		m.Push("span_time", kit.Format(d/3600)+"小时")
	})
	return m
}
func (s tencentmeeting) MeetingPost(m *ice.Message, arg ...string) *ice.Message {
	quick := true
	for i := 0; i < len(arg); i += 2 {
		kit.If(strings.HasSuffix(arg[i], "_time"), func() {
			arg[i+1] = kit.Format(kit.Time(arg[i+1]) / int64(time.Second))
			quick = false
		})
	}
	arg = append(arg, "type", kit.Select("0", "1", quick))
	return s.spide(m, http.MethodPost, "meetings", "userid", "$user_id", "instanceid", "1", arg)
}
func (s tencentmeeting) MeetingPut(m *ice.Message, arg ...string) *ice.Message {
	quick := true
	for i := 0; i < len(arg); i += 2 {
		kit.If(strings.HasSuffix(arg[i], "_time"), func() {
			arg[i+1] = kit.Format(kit.Time(arg[i+1]) / int64(time.Second))
			quick = false
		})
	}
	arg = append(arg, "type", kit.Select("0", "1", quick))
	return s.spide(m, http.MethodPut, kit.Format("meetings/%s", m.Option("meeting_id")), "userid", "$user_id", "instanceid", 1, arg)
}
func (s tencentmeeting) MeetingCancel(m *ice.Message, arg ...string) *ice.Message {
	return s.spide(m, http.MethodPost, kit.Format("meetings/%s/cancel", m.Option("meeting_id")), "userid", "$user_id", "instanceid", 1, "reason_code", 1)
}
func init() { ice.TeamCtxCmd(tencentmeeting{}) }

func (s tencentmeeting) spide(m *ice.Message, method, api string, arg ...ice.Any) *ice.Message {
	msg := s.Select(m.Spawn(), m.OptionSimple(model.AUTH_UID)...)
	if msg.Length() == 0 {
		m.Echo("请联系公司管理员，配置腾讯会议的账号")
		return m
	}
	for i := 0; i < len(arg)-1; i += 2 {
		kit.If(arg[i+1] == "$user_id", func() { arg[i+1] = kit.Select(msg.Append(model.USER_ID), m.Option(model.USER_ID)) })
	}
	method = kit.Select(http.MethodGet, method)
	uri, body, args := kit.MergeURL("/v1/"+api, arg...), "", []string{}
	nonce := kit.Format(uint64(100000 + rand.New(rand.NewSource(time.Now().UnixNano())).Intn(900000)))
	now := kit.Format(time.Now().Unix())
	kit.If(kit.IsIn(method, http.MethodPost, http.MethodPut), func() {
		uri, body = kit.MergeURL("/v1/"+api), kit.Format(kit.Dict(arg...))
		args = []string{web.SPIDE_DATA, body}
	})
	m.Options(web.SPIDE_HEADER, map[string]string{
		"AppId": msg.Append("app_id"), "SdkId": msg.Append("sdk_id"), "X-TC-Key": msg.Append("secret_id"),
		"X-TC-Nonce": nonce, "X-TC-Timestamp": now, "X-TC-Registered": "1",
		"X-TC-Signature": doSignature(msg.Append("secret_id"), msg.Append("secret_key"), method, nonce, now, uri, body),
	}).Cmdy(web.SPIDE, ice.DEV, web.SPIDE_RAW, method, "https://api.meeting.qq.com"+uri, args)
	kit.If(kit.IsIn(method, http.MethodPost, http.MethodPut), func() { kit.If(!m.IsErr(), func() { m.ProcessRefresh() }) })
	return m
}
func (s tencentmeeting) parseList(m *ice.Message, key string, arg ...string) *ice.Message {
	data := kit.UnMarshal(m.Result())
	if m.Warn(kit.Value(data, "error_info.error_code") != nil, kit.Value(data, "error_info.message")) {
		return m
	}
	kit.For(kit.Value(data, key), func(value ice.Map) {
		kit.For(arg, func(key string) {
			if strings.HasSuffix(key, "_time") {
				m.Push(key, strings.TrimSuffix(time.Unix(kit.Int64(value[key]), 0).Format(ice.MOD_TIME), ":00.000"))
			} else {
				m.Push(key, value[key])
			}
		})
	})
	return m.SetResult()
}
func doSignature(secretId string, secretKey string, method string, nonce string, now string, uri string, body string) string {
	h := hmac.New(sha256.New, []byte(secretKey))
	h.Write([]byte(fmt.Sprintf("%s\n%s\n%s\n%s", method, fmt.Sprintf("X-TC-Key=%s&X-TC-Nonce=%s&X-TC-Timestamp=%s", secretId, nonce, now), uri, body)))
	return base64.StdEncoding.EncodeToString([]byte(hex.EncodeToString(h.Sum([]byte{}))))
}
