package weixinpayment

import (
	"strings"

	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/api"
	"shylinux.com/x/community/src/gonganxitong"
	"shylinux.com/x/community/src/renzhengshouquan"
	"shylinux.com/x/community/src/renzhengshouquan/external/weixinpayment/model"

	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/services/payments/jsapi"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
)

type weixinpayment struct {
	renzhengshouquan.Table
	order    string `data:"11"`
	role     string `data:"leader"`
	fields   string `data:"user_uid,app_id,mch_id,api_key,cert_no,cert_key"`
	create   string `name:"create app_id* mch_id* api_key* cert_no* cert_key*:textarea" role:"leader"`
	remove   string `name:"remove" role:"leader"`
	list     string `name:"list auth_uid uid auto" role:"leader"`
	response string `name:"response" role:"void"`
}

func (s weixinpayment) Init(m *ice.Message, arg ...string) {
	gonganxitong.PaymentVendorAdd(m.PrefixKey(), s)
	s.Table.Init(m, arg...)
}
func (s weixinpayment) List(m *ice.Message, arg ...string) {
	s.ValueList(m, arg).Display("")
	kit.If(m.Length() > 0, func() { m.Action() })
}
func (s weixinpayment) Request(m *ice.Message, arg ...string) {
	m.Option("auth_uid", "308a1ca57c40e2a46f6568e017269cdd")
	msg := s.Select(m.Spawn(), m.OptionSimple(model.AUTH_UID)...)
	svc := jsapi.JsapiApiService{Client: s.newClient(m, msg)}
	resp, result, err := svc.PrepayWithRequestPayment(m, jsapi.PrepayRequest{
		Appid:       core.String(msg.Append("app_id")),
		Mchid:       core.String(msg.Append("mch_id")),
		Description: core.String(m.Option(model.TITLE)),
		Attach:      core.String(m.Option(model.CONTENT)),
		OutTradeNo:  core.String(m.Option(model.UID)),
		Amount:      &jsapi.Amount{Total: core.Int64(kit.Int64(m.Option(model.AMOUNT)))},
		Payer:       &jsapi.Payer{Openid: core.String(m.Cmd(api.GONGANXITONG_USER, s.Select, model.UID, m.Option(model.USER_UID)).Append(model.OPEN_ID))},
		NotifyUrl:   core.String(strings.Split(m.MergePodCmd("", m.PrefixKey()), "?")[0] + "/action/response/" + m.Option(model.PLACE_UID) + "/" + m.Option(model.UID)),
	})
	if !m.Warn(err) {
		kit.For(kit.UnMarshal(kit.Format(resp)), func(k, v string) { m.Push(kit.LowerCapital(k), v) })
	}
	m.Info("what %v %v %v", resp, result, err)
}
func (s weixinpayment) Response(m *ice.Message, arg ...string) {
	if len(arg) > 1 {
		if m.Option("event_type") == "TRANSACTION.SUCCESS" {
			m.Cmdy(api.GONGANXITONG_PAYMENTLIST, s.Response, arg)
		}
	}
}

func init() { ice.TeamCtxCmd(weixinpayment{}) }

func (s weixinpayment) newClient(m, msg *ice.Message) *core.Client {
	if key, err := utils.LoadPrivateKey(msg.Append("cert_key")); m.Warn(err) {
		return nil
	} else if client, err := core.NewClient(m, option.WithWechatPayAutoAuthCipher(msg.Append("mch_id"), msg.Append("cert_no"), key, msg.Append("api_key"))); m.Warn(err) {
		return nil
	} else {
		return client
	}
}
